@@ -1,9 +1,10 @@
-# $Id: String.pm,v 1.45 2003/04/22 23:25:42 vipul Exp $
+# $Id: String.pm,v 1.48 2005/06/13 21:09:59 vipul Exp $
 package Razor2::String;
 
 use Digest::SHA1 qw(sha1_hex);
 use URI::Escape;
 use Razor2::Preproc::enBase64;
+use Data::Dumper;
 
 #use MIME::Parser;
 
@@ -838,7 +839,16 @@ sub split_mime {
         print "bad mime body [len=". length($body) ."], not doing \\n--boundary, fixed tho.\n" if $debug > 1;
         $body = "garbage\n$body";  
     }
-    my @tmpparts = split /\n\Q--$boundary\E\r*\n/, $body;
+    my @tmpparts;
+    unless ($body =~ /\Q--$boundary\E/) { 
+        # Sometimes there's a boundary in the headers 
+        # but none in email.  In such cases, we'll 
+        # treat the entire body as a part.
+        push @tmpparts, "garbage";
+        push @tmpparts, $body;
+    } else { 
+        @tmpparts = split /\n\Q--$boundary\E\r*\n/, $body;
+    }
     shift @tmpparts;  # trash everything up to the first boundary;
     foreach (@tmpparts) {
         # perhaps we should add a header based on default content-type? 
@@ -890,12 +900,14 @@ sub prep_part {
         $hdr  = "X-Razor2-Origlen-Body: $len\n" . $hdr;
         #print "maxbody=$maxbody body went from $len to ". length($body) ."\n";
     }
+    
     if ((my $len = length($hdr)) > $maxheader) {
         $hdr  = "X-Razor2-Origlen-Header: $len\n" . $hdr;
-        while (length($hdr) > $maxheader) {
-            $hdr =~ s/.*\n$//;  # remove last line of headers
+        if (length($hdr) > $maxheader) { 
+            $hdr = substr $hdr, 0, $maxheader; 
+            $hdr =~ s/([^\n]+)$//s;  # remove last, incomplete line
         }
-        #print "maxhdr=$maxheader header went from $len to ". length($hdr) ."\n";
+        # print "maxhdr=$maxheader header went from $len to ". length($hdr) ."\n";
     }
 
     my $dude = "$hdr\n$body";
@@ -939,8 +951,9 @@ sub prep_mail {
 
     if ((my $len = length($orig_hdr)) > $maxorighdr) {
         $hdr  = "X-Razor2-Origlen-Header: $len\n" . $orig_hdr;
-        while (length($hdr) > $maxorighdr) {
-            $hdr =~ s/.*\n$//;  # remove last line of headers
+        if (length($hdr) > $maxorighdr) { 
+            $hdr = substr $hdr, 0, $maxorighdr; 
+            $hdr =~ s/([^\n]+)$//s;  # remove last, incomplete line
         }
         #print "max=$maxorighdr orig_header went from $len to ". length($hdr) ."\n";
         $orig_hdr = $hdr;
@@ -980,17 +993,13 @@ print "    Part: ", $ent->bodyhandle->path, " (", scalar($ent->head->mime_type),
 # output: hash ref or array containg bits that are set
 #         2D == (1, 3, 4, 6)
 
-sub hexbits2hash {
+sub hexbits2hash { 
+
     my $hex = shift;
-
     my %h;
-    my $i = 0;
-    foreach (reverse split '', unpack "B*", pack "H*", $hex) {
-        $i++;
-        $h{$i} = 1 if  $_ eq 1;
-    }
-
+    for (0..31) { if (hex($hex) & (2**31)>>(31-$_)) { $h{$_+1} = 1 } }
     return wantarray ? (sort keys %h) : \%h;
+
 }
 
 
